#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
"""
Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

"""
import os
from pygraph.classes.graph import graph
from pygraph.algorithms.heuristics import chow
from pygraph.algorithms.heuristics import euclidean
from pygraph.algorithms.minmax import heuristic_search
from pygraph.classes.exceptions import AdditionError
import globals
import time

class field:
    def __init__(self,path):
        self.path = path

    def readField(self):
        filelines = open(self.path).readlines()

        self.field = []
        self.knownWays = []

        for line in filelines:
            self.field.append([])
            for field in list(line[:-1]):
                self.field[-1].append({"val" : field, "toxic" : 0 , "mag" : 0})
                if field == "#":
                    self.field[-1][-1]["mag"] = 0.1

        self.buildGraph()
       
    def buildGraph(self):
        self.gr = graph()

        for lIndex,line in enumerate(self.field):
            for sIndex,spot in enumerate(line):
                if self.NoWall(lIndex,sIndex) == True:
                    self.gr.add_node(self.getNodeName(lIndex,sIndex))
                    self.gr.add_node_attribute(self.getNodeName(lIndex,sIndex),('position',(lIndex,sIndex)))


        for lIndex,line in enumerate(self.field):
            for sIndex,spot in enumerate(line):
                if lIndex != 0 and lIndex != len(self.field) -1 and sIndex != 0 and sIndex != len(line) - 1 and self.NoWall(lIndex,sIndex) == True: 
                    #print lIndex,sIndex
                    #North
                    self.addMainEdge(lIndex,sIndex,lIndex -1 ,sIndex)
                    #North East 
                    self.addDiagonalEdge(lIndex,sIndex,lIndex - 1, sIndex + 1)
                    #East
                    self.addMainEdge(lIndex,sIndex,lIndex,sIndex + 1)
                    #South East
                    self.addDiagonalEdge(lIndex,sIndex,lIndex + 1, sIndex + 1)
                    #South
                    self.addMainEdge(lIndex,sIndex,lIndex + 1,sIndex)
                    #Sout West
                    self.addDiagonalEdge(lIndex,sIndex,lIndex + 1, sIndex - 1)
                    #West
                    self.addMainEdge(lIndex,sIndex,lIndex,sIndex - 1)
                    #North West
                    self.addDiagonalEdge(lIndex,sIndex,lIndex - 1, sIndex - 1)

    def addMainEdge(self,lIndex,sIndex,lTargetIndex,sTargetIndex):
        if self.NoWall(lTargetIndex,sTargetIndex) == True:
            try:
                #print "Adding edge from (%d,%d) is %s to (%d,%d) is %s" % (lIndex,sIndex,self.field[lIndex][sIndex]["val"] ,lTargetIndex,sTargetIndex,self.field[lTargetIndex][sTargetIndex]["val"] )
                #print "Adding edge from (%s) to (%s)" % (self.getNodeName(lIndex,sIndex),self.getNodeName(lTargetIndex,sTargetIndex))
                
                self.gr.add_edge((self.getNodeName(lIndex,sIndex),self.getNodeName(lTargetIndex,sTargetIndex)))
            except AdditionError:
                pass
        
    def addDiagonalEdge(self,lIndex,sIndex,lTargetIndex,sTargetIndex):
        if self.NoWall(lTargetIndex,sTargetIndex) == True and self.field[lIndex][sTargetIndex]["val"] != "#" and self.field[lTargetIndex][sIndex]["val"] != "#" :
            try:
                self.gr.add_edge((self.getNodeName(lIndex,sIndex),self.getNodeName(lTargetIndex,sTargetIndex)))
            except AdditionError:
                pass

    def NoWall(self,lIndex,sIndex):
        return self.field[lIndex][sIndex]["val"] != "#"

    def getNodeName(self,lIndex,sIndex):
        return "%03d_%03d" % (lIndex,sIndex)

    def setToxic(self,x,y,value):

        self.field[y][x] = value

    def __str__(self):
        str = ""
        for index,line in enumerate(self.field):
            str += " %02d " % index
            for field in line:
                str += field["val"]
            str += "\n"
        return str
    
    def moveFromTo(self,start,target):
        #searcher =  chow.chow()
        searcher =  euclidean.euclidean()
        startt = time.time()
        searcher.optimize(self.gr)
        end = time.time()
        print "Optimizing %f" % (end-startt)

        startt = time.time()
        result = heuristic_search(self.gr,self.getNodeName(start.x,start.y),self.getNodeName(target.x,target.y),searcher)
        end = time.time()

        print "Route %f" % (end-startt)

        print result

if __name__ == "__main__":
    test = field("../freiesmagazin-contest3/maps/board1.map")
    test.readField()

    print test
    print len(test.gr.nodes())
    print len(test.gr.edges())

    start = globals.position()
    target = globals.position()

    start.x = 1
    start.y = 1

    target.x = 2
    target.y = 20

    test.moveFromTo(start,target)
